"""Command is run and output is analyzed"""

from BasicRunner import BasicRunner
from PyFoam.LogAnalysis.FoamLogAnalyzer import FoamLogAnalyzer
from os import path,mkdir

class AnalyzedRunner(BasicRunner):
    """The output of a command is analyzed while being run

    Side effects (files written etc) depend on the analyzer"""
    
    def __init__(self,analyzer,argv=None,silent=False,logname="PyFoamSolve"):
        """ analyzer - the analyzer for the output
        argv, silent, logname - see BasicRunner"""
        BasicRunner.__init__(self,argv,silent,logname)

        self.analyzer=analyzer
        self.logDir=path.join(self.dir,logname+".analyzed")
        
        if not path.exists(self.logDir):
            mkdir(self.logDir)
        
        self.reset()
        
    def lineHandle(self,line):
        """Not to be called: calls the analyzer for the current line"""
        self.analyzer.analyzeLine(line)
        
    def reset(self):
        """reset the analyzer"""
        self.analyzer.setDirectory(self.logDir)

    def getDirname(self):
        """Get the name of the directory where the data is written to"""
        return self.logDir
    
    def getTime(self):
        """Get the execution time"""
        return self.analyzer.getTime()
    
