"""Stop solver at convergence"""

from os import path

from AnalyzedRunner import AnalyzedRunner
from PyFoam.LogAnalysis.SteadyConvergedLineAnalyzer import SteadyConvergedLineAnalyzer
from PyFoam.RunDictionary.ParameterFile import ParameterFile

class ConvergenceRunner(AnalyzedRunner):
    """It is assumed that the provided solver is a steady state
    solver. After all the linear solvers have initial residuals below
    their limits the run is assumed to be convergent and the run is
    stopped by setting

    stopAt nextWrite;
    writeInterval 1;

    in the controlDict"""
    
    def __init__(self,analyzer,argv=None,silent=False,logname="PyFoamSolve"):
        """See AnalyzedRunner"""
        AnalyzedRunner.__init__(self,analyzer,argv,silent,logname)
        
        self.analyzer.addAnalyzer("Convergence",SteadyConvergedLineAnalyzer())

        self.controlDict=ParameterFile(path.join(self.dir,"system","controlDict"),backup=True)
        
    def lineHandle(self,line):
        """Not to be called: Stops the solver"""
        AnalyzedRunner.lineHandle(self,line)

        if not self.analyzer.goOn():
            dt=self.controlDict.readParameter("deltaT")
            self.controlDict.replaceParameter("stopAt","nextWrite")
            self.controlDict.replaceParameter("writeInterval",dt)
            
    def stopHandle(self):
        """Not to be called: Restores controlDict"""
        AnalyzedRunner.stopHandle(self)

        self.controlDict.restore()
        
