"""Analyzes lines with regular expressions"""

import re

from FileLineAnalyzer import FileLineAnalyzer

class RegExpLineAnalyzer(FileLineAnalyzer):
    """Parses lines for an arbitrary regular expression

    Only one data-set is stored per time-step

    One pattern group of the RegExp can be used as a unique
    identifier, so that more than one data-sets can be stored per
    time-step"""
    
    def __init__(self,name,exp,idNr=None,titles=[]):
        """
        @param name: name of the expression (needed for output
        @param exp: the regular expression
        @param idNr: number of the pattern group that is used as an identifier
        @param titles: titles of the columns
        """
        FileLineAnalyzer.__init__(self,titles=titles)

        self.name=name
        self.idNr=idNr
        
        self.strExp=exp
        self.exp=re.compile(self.strExp)

        self.data={}
        
    def doAnalysis(self,line):
        """Analyzes line and stores and writes the data"""
        tm=self.parent.getTime()
        m=self.exp.match(line)
        if m!=None:
            name=self.name
            fdata=m.groups()
            if self.idNr!=None:
                ID=m.group(self.idNr)
                name+="_"+ID
                fdata=fdata[:self.idNr-1]+fdata[self.idNr:]
            else:
                ID=""

            self.sub(ID)[float(tm)]=fdata
            if ID!="":
                self.sub("")[float(tm)]=m.groups()
                
            self.files.write(name,tm,fdata)

    def sub(self,ID):
        """ get the data set for the identifier ID"""
        if not self.data.has_key(ID):
            self.data[ID]={}
        return self.data[ID]
    
    def getTimes(self,ID=None):
        """get the available time for the identifier ID"""
        if ID==None:
            ID=""
        return self.sub(ID).keys()

    def getIDs(self):
        """get a list of the available IDs"""
        ids=self.data.keys()
        if "" in ids:
            ids.remove("")
        return ids
    
    def getLast(self,ID=None):
        """get the last time for the identifier ID"""
        times=self.getTimes(ID)
        if len(times)>0:
            return max(times)
        else:
            return None
        
    def getData(self,time=None,ID=None):
        """get a data value at a specific time for a specific ID"""
        if ID==None:
            ID=""

        if time==None:
            time=self.getLast(ID)
        else:
            time=float(time)
            
        data=self.sub(ID)
        
        if data.has_key(time):
            return data[time]
        else:
            return None
        
